#!/bin/bash

#=================================================
# COMMON VARIABLES
#=================================================

# dependencies used by the app
pkg_dependencies="postgresql postgresql-contrib redis-server"

nodejs_version=14

#=================================================
# PERSONAL HELPERS
#=================================================

#=================================================
# EXPERIMENTAL HELPERS
#=================================================


MONGO_DEBIAN_SERVICENAME="mongodb"
MONGO_CE_SERVICENAME="mongod"
MONGO_DEBIAN_DEPENDENCIES="mongodb mongodb-server mongo-tools"
MONGO_CE_DEPENDENCIES="mongodb-org mongodb-org-server mongodb-org-tools"
MONGO_DEBIAN_CONFIG="/etc/mongodb.conf"
MONGO_CE_CONFIG="/etc/mongod.conf"
MONGO_CE_REPO="deb http://repo.mongodb.org/apt/debian buster/mongodb-org/4.4 main"
MONGO_CE_KEY="https://www.mongodb.org/static/pgp/server-4.4.asc"

# Execute a mongo command
#
# example: ynh_mongo_exec --command='db.getMongo().getDBNames().indexOf("wekan")'
# example: ynh_mongo_exec --command="db.getMongo().getDBNames().indexOf(\"wekan\")"
#
# usage: ynh_mongo_exec [--user=user] [--password=password] [--authenticationdatabase=authenticationdatabase] [--database=database] [--host=host] [--port=port] --command="command" [--eval]
# | arg: -u, --user=                      - The user name to connect as
# | arg: -p, --password=                  - The user password
# | arg: -d, --authenticationdatabase=    - The authenticationdatabase to connect to
# | arg: -d, --database=                  - The database to connect to
# | arg: -h, --host=                      - The host to connect to
# | arg: -P, --port=                      - The port to connect to
# | arg: -c, --command=                   - The command to evaluate
# | arg: -e, --eval                       - Evaluate instead of execute the command.
#
#
ynh_mongo_exec() {
    # Declare an array to define the options of this helper.
    local legacy_args=upadhPce
    local -A args_array=( [u]=user= [p]=password= [a]=authenticationdatabase= [d]=database= [h]=host= [P]=port= [c]=command= [e]=eval )
    local user
    local password
    local authenticationdatabase
    local database
    local host
    local port
    local command
    local eval
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"
    user="${user:-}"
    password="${password:-}"
    authenticationdatabase="${authenticationdatabase:-}"
    database="${database:-}"
    host="${host:-}"
    port="${port:-}"
    eval=${eval:-0}

    # If user is provided
    if [ -n "$user" ]
    then
        user="--username=$user"
        
        # If password is provided
        if [ -n "$password" ]
        then
            password="--password=$password"
        fi

        # If authenticationdatabase is provided
        if [ -n "$authenticationdatabase" ]
        then
            authenticationdatabase="--authenticationDatabase=$authenticationdatabase"
        else
            authenticationdatabase="--authenticationDatabase=admin"
        fi
    else
        password=""
        authenticationdatabase=""
    fi

    # If host is provided
    if [ -n "$host" ]
    then
        host="--host=$host"
    fi

    # If port is provided
    if [ -n "$port" ]
    then
        port="--port=$port"
    fi

    # If eval is not provided
    if [ $eval -eq 0 ]
    then
        # If database is provided
        if [ -n "$database" ]
        then
            database="use $database"
        else
            database=""
        fi

        mongo --quiet $user $password $authenticationdatabase $host $port <<EOF
$database
${command}
quit()
EOF
    else
        # If database is provided
        if [ -n "$database" ]
        then
            database="$database"
        else
            database=""
        fi
        
        mongo --quiet $database $user $password $authenticationdatabase $host $port --eval="$command"
    fi
}

# Drop a database
#
# [internal]
#
# If you intend to drop the database *and* the associated user,
# consider using ynh_mongo_remove_db instead.
#
# usage: ynh_mongo_drop_db --database=database
# | arg: -d, --database=    - The database name to drop
#
#
ynh_mongo_drop_db() {
    # Declare an array to define the options of this helper.
    local legacy_args=d
    local -A args_array=( [d]=database= )
    local database
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    ynh_mongo_exec --database="$database" --command='db.runCommand({dropDatabase: 1})'
}

# Dump a database
#
# example: ynh_mongo_dump_db --database=wekan > ./dump.bson
#
# usage: ynh_mongo_dump_db --database=database
# | arg: -d, --database=    - The database name to dump
# | ret: the mongodump output
#
#
ynh_mongo_dump_db() {
    # Declare an array to define the options of this helper.
    local legacy_args=d
    local -A args_array=( [d]=database= )
    local database
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    mongodump --quiet --db="$database" --archive
}

# Create a user
#
# [internal]
#
# usage: ynh_mongo_create_user --db_user=user --db_pwd=pwd --db_name=name
# | arg: -u, --db_user=    - The user name to create
# | arg: -p, --db_pwd=     - The password to identify user by
# | arg: -n, --db_name=    - Name of the database to grant privilegies
#
#
ynh_mongo_create_user() {
    # Declare an array to define the options of this helper.
    local legacy_args=unp
    local -A args_array=( [u]=db_user= [n]=db_name= [p]=db_pwd= )
    local db_user
    local db_name
    local db_pwd
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    # Create the user and set the user as admin of the db
    ynh_mongo_exec --database="$db_name" --command='db.createUser( { user: "'${db_user}'", pwd: "'${db_pwd}'", roles: [ { role: "readWrite", db: "'${db_name}'" } ] } );'
    
    # Add clustermonitoring rights
    ynh_mongo_exec --database="$db_name" --command='db.grantRolesToUser("'${db_user}'",[{ role: "clusterMonitor", db: "admin" }]);'
}

# Check if a mongo database exists
#
# usage: ynh_mongo_database_exists --database=database
# | arg: -d, --database=    - The database for which to check existence
# | exit: Return 1 if the database doesn't exist, 0 otherwise
#
#
ynh_mongo_database_exists() {
    # Declare an array to define the options of this helper.
    local legacy_args=d
    local -A args_array=([d]=database=)
    local database
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    if [ $(ynh_mongo_exec --command='db.getMongo().getDBNames().indexOf("'${database}'")' --eval) -lt 0 ]
    then
        return 1
    else
        return 0
    fi
}

# Restore a database
#
# example: ynh_mongo_restore_db --database=wekan < ./dump.bson
#
# usage: ynh_mongo_restore_db --database=database
# | arg: -d, --database=    - The database name to restore
#
#
ynh_mongo_restore_db() {
    # Declare an array to define the options of this helper.
    local legacy_args=d
    local -A args_array=( [d]=database= )
    local database
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    mongorestore --quiet --db="$database" --archive
}

# Drop a user
#
# [internal]
#
# usage: ynh_mongo_drop_user --db_user=user --db_name=name
# | arg: -u, --db_user=    - The user to drop
# | arg: -n, --db_name=    - Name of the database
#
#
ynh_mongo_drop_user() {
    # Declare an array to define the options of this helper.
    local legacy_args=un
    local -A args_array=( [u]=db_user= [n]=db_name= )
    local db_user
    local db_name
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    ynh_mongo_exec --database="$db_name" --command='db.dropUser("'$db_user'", {w: "majority", wtimeout: 5000})'
}

# Create a database, an user and its password. Then store the password in the app's config
#
# usage: ynh_mongo_setup_db --db_user=user --db_name=name [--db_pwd=pwd]
# | arg: -u, --db_user=    - Owner of the database
# | arg: -n, --db_name=    - Name of the database
# | arg: -p, --db_pwd=     - Password of the database. If not provided, a password will be generated
#
# After executing this helper, the password of the created database will be available in $db_pwd
# It will also be stored as "mongopwd" into the app settings.
#
#
ynh_mongo_setup_db() {
    # Declare an array to define the options of this helper.
    local legacy_args=unp
    local -A args_array=( [u]=db_user= [n]=db_name= [p]=db_pwd= )
    local db_user
    local db_name
    db_pwd=""
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    local new_db_pwd=$(ynh_string_random) # Generate a random password
    # If $db_pwd is not provided, use new_db_pwd instead for db_pwd
    db_pwd="${db_pwd:-$new_db_pwd}"
    
    # Create the user and grant access to the database
    ynh_mongo_create_user --db_user="$db_user" --db_pwd="$db_pwd" --db_name="$db_name"

    # Store the password in the app's config
    ynh_app_setting_set --app=$app --key=db_pwd --value=$db_pwd 
}

# Remove a database if it exists, and the associated user
#
# usage: ynh_mongo_remove_db --db_user=user --db_name=name
# | arg: -u, --db_user=    - Owner of the database
# | arg: -n, --db_name=    - Name of the database
#
#
ynh_mongo_remove_db() {
    # Declare an array to define the options of this helper.
    local legacy_args=un
    local -A args_array=( [u]=db_user= [n]=db_name= )
    local db_user
    local db_name
    # Manage arguments with getopts
    ynh_handle_getopts_args "$@"

    if ynh_mongo_database_exists --database=$db_name; then  # Check if the database exists
        ynh_mongo_drop_db --database=$db_name  # Remove the database
    else
        ynh_print_warn --message="Database $db_name not found"
    fi

    # Remove mongo user if it exists
    ynh_mongo_drop_user --db_user=$db_user --db_name=$db_name
}

# Install MongoDB and integrate MongoDB service in YunoHost
#
# usage: ynh_install_mongo
#
#
ynh_install_mongo() {
    ynh_print_info --message="Installing MongoDB..."

    # Define Mongo Service Name
    if $(dpkg --compare-versions $(cat /etc/debian_version) gt "10.0")
    then
        ynh_print_info --message="Installing MongoDB Community Edition..."
        ynh_install_extra_app_dependencies --repo="$MONGO_CE_REPO" --package="$MONGO_CE_DEPENDENCIES" --key="$MONGO_CE_KEY"
        MONGODB_SERVICENAME=$MONGO_CE_SERVICENAME
    else
        ynh_print_info --message="Installing MongoDB Debian..."
        ynh_install_app_dependencies $MONGO_DEBIAN_DEPENDENCIES
        MONGODB_SERVICENAME=$MONGO_DEBIAN_SERVICENAME
    fi
    mongodb_servicename=$MONGODB_SERVICENAME
    
    # Make sure MongoDB is started and enabled
    systemctl is-enabled $MONGODB_SERVICENAME -q || systemctl enable $MONGODB_SERVICENAME --quiet
    systemctl is-active $MONGODB_SERVICENAME -q || ynh_systemd_action --service_name=$MONGODB_SERVICENAME --action=restart --line_match="aiting for connections" --log_path="/var/log/mongodb/$MONGODB_SERVICENAME.log"
    
    # Integrate MongoDB service in YunoHost
    yunohost service add $MONGODB_SERVICENAME --description="MongoDB daemon" --log="/var/log/mongodb/$MONGODB_SERVICENAME.log"
}

# Remove MongoDB
# Only remove the MongoDB service integration in YunoHost for now
# if MongoDB package as been removed
#
# usage: ynh_remove_mongo
#
#
ynh_remove_mongo() {
    # Only remove the mongodb service if it is not installed.
    if ! ynh_package_is_installed --package="mongodb*"
    then
        ynh_print_info --message="Removing MongoDB service..."
        # Define Mongo Service Name
        if [ "$(lsb_release --codename --short)" = "buster" ]; then
            MONGODB_SERVICENAME=$MONGO_CE_SERVICENAME
        else
            MONGODB_SERVICENAME=$MONGO_DEBIAN_SERVICENAME
        fi
        # Remove the mongodb service
        yunohost service remove $MONGODB_SERVICENAME
        # ynh_secure_remove --file=$MONGO_ROOT_PWD_FILE
    fi
}


#=================================================
# FUTURE OFFICIAL HELPERS
#=================================================

# Send an email to inform the administrator
#
# usage: ynh_send_readme_to_admin app_message [recipients]
# | arg: app_message - The message to send to the administrator.
# | arg: recipients - The recipients of this email. Use spaces to separate multiples recipients. - default: root
#	example: "root admin@domain"
#	If you give the name of a YunoHost user, ynh_send_readme_to_admin will find its email adress for you
#	example: "root admin@domain user1 user2"
ynh_send_readme_to_admin() {
	local app_message="${1:-...No specific information...}"
	local recipients="${2:-root}"

	# Retrieve the email of users
	find_mails () {
		local list_mails="$1"
		local mail
		local recipients=" "
		# Read each mail in argument
		for mail in $list_mails
		do
			# Keep root or a real email address as it is
			if [ "$mail" = "root" ] || echo "$mail" | grep --quiet "@"
			then
				recipients="$recipients $mail"
			else
				# But replace an user name without a domain after by its email
				if mail=$(ynh_user_get_info "$mail" "mail" 2> /dev/null)
				then
					recipients="$recipients $mail"
				fi
			fi
		done
		echo "$recipients"
	}
	recipients=$(find_mails "$recipients")

	local mail_subject="☁️🆈🅽🅷☁️: \`$app\` was just installed!"

	local mail_message="This is an automated message from your beloved YunoHost server.

Specific information for the application $app.

$app_message

---
Automatic diagnosis data from YunoHost

$(yunohost tools diagnosis | grep -B 100 "services:" | sed '/services:/d')"

	# Define binary to use for mail command
	if [ -e /usr/bin/bsd-mailx ]
	then
		local mail_bin=/usr/bin/bsd-mailx
	else
		local mail_bin=/usr/bin/mail.mailutils
	fi

	# Send the email to the recipients
	echo "$mail_message" | $mail_bin -a "Content-Type: text/plain; charset=UTF-8" -s "$mail_subject" "$recipients"
}

